%% GW_USW_Joints_metrics_2.m
% This script is under an MIT License.
% Copyright (c) 2018 Pedro Andre Viegas Ochoa de Carvalho
% The licence can be found in the folder:
%   Ultrasonic_guided_wave_and_single-lap_shear_tests_in_defective_ultrasonically_welded_thermoplastic_composite_joints

close all
clear
clc

disp('Analysis of GW propagation in ultrasonically welded joints');
disp(' ');

%% Check existence of test result
answer = '0';
if exist('GW_test_results_2.mat','file') == 2
    quest_str = {'Test results 2 already exist';...
        'Do you want to generate them again?'};
    answer = questdlg(quest_str,'Test Results','Yes','No','Cancel','No');
end

if strcmp(answer,'Cancel') == 1
    return
end

%% Some control parameters
opt_filter = 'narrow';
bd_lev = 3; % dB
psm_ord = 1;

if strcmp(answer,'Yes') == 1 || exist('GW_test_results_2.mat','file') == 0
    %% Select folder of data files
    curdir = pwd;
    match_name = [curdir,'\**/*uw*.mat'];
    file_list = dir(match_name);
    
    %% Memory allocation
    metrics = zeros(14,6,4);
    plot_notes = cell(14,1);
    
    %%
    for f = 1:length(file_list)
        
        %% Load each file in the list
        fdir = file_list(f).folder;
        fname = file_list(f).name;
        load(fullfile(fdir,fname));
        disp(fname);
        
        %% Extract information from file string
        % Frequency
        p = strfind(fname,'kHz');
        if isempty(p) == 0
            freq_str = fname(p-3:p-1);
            freq = str2num(freq_str);
        end
        % Specimen name
        p = strfind(fname,'_uw');
        if isempty(p) == 0
            spec_name = fname(p+1:p+12);
        end
        
        %% Extract signals
        t0 = Tstart;
        dt = Tinterval;
        sigL = RequestedLength;
        t = (t0:dt:(t0 + ((sigL - 1) * dt)))';
        x_act = A(1:sigL);
        x_sen = B(1:sigL);
        % Free memory
        clearvars A B Tstart Tinterval RequestedLength
        
        %% Compute quantities for analysis
        %%% Time-domain metrics
        % Time-of-Flight (approach 2)
        [~,idx_a] = max(x_act);
        [~,idx_s] = max(x_sen);
        tof = t(idx_s) - t(idx_a);
        
        % Signal energy
        es_t = trapz(t,abs(x_sen).^2);
        
        %%% Frequency-domain metrics
        fft_x = abs(fftshift(fft(x_sen)));
        fft_xa = abs(fftshift(fft(x_act)));
        Fs = 1/dt;
        dF = Fs/sigL;
        f_vec = -Fs/2:dF:Fs/2-dF;
        ki = find(f_vec >= 0,1); % Find index of freq = 0
        kf = find(f_vec >= 1e6,1); % Find index of freq = 1 MHz
        fft_x = fft_x(ki:kf);
        fft_xa = fft_xa(ki:kf);
        f_vec = f_vec(ki:kf);
        
        % Characteristic frequency shift
        fpk = sum(fft_x' .* f_vec) / sum(fft_x);
        fpk_a = sum(fft_xa' .* f_vec) / sum(fft_xa);
        dfpk = fpk - fpk_a;
        
        % FFT peak frequency
        [max_fft,idx] = max(fft_x);
        fft_fpk = f_vec(idx);
        [max_ffta,idx_a] = max(fft_xa);
        fft_fpk_a = f_vec(idx_a);
        dfft_fpk = fft_fpk - fft_fpk_a;
        
        % Power spectral moment
        fnyq = 0.5 * Fs;
        [psd,f_psd] = pwelch(x_sen,[],[],[],Fs);
        k = find(f_psd >=0 & f_psd <= fnyq);
        f_psd = f_psd(k);
        psd = psd(k);
        psm = trapz(f_psd,(f_psd - fpk).^psm_ord .* psd);
        
        % Frequency bandwidth
        [env_fft,~] = envelope(fft_x,5,'peak');
        bw_lev = -25; % dB
        L1 = [f_vec;...
           env_fft'];
        L2 = [f_vec;...
           db2mag(bw_lev)*max(fft_x)*ones(1,length(f_vec))];
        pts = InterX(L1,L2);
        bw = abs(pts(1,1) - pts(1,end));
        
        %% Define frequency index
        switch freq
            case 204
                f_idx = 1;
            case 349
                f_idx = 2;
            case 486
                f_idx = 3;
            case 619
                f_idx = 4;
        end
        
        %% Store results
        metrics(specimen_idx(spec_name),1,f_idx) = dfft_fpk;
        metrics(specimen_idx(spec_name),2,f_idx) = es_t;
        metrics(specimen_idx(spec_name),3,f_idx) = tof;
        metrics(specimen_idx(spec_name),4,f_idx) = dfpk;
        metrics(specimen_idx(spec_name),5,f_idx) = psm;
        metrics(specimen_idx(spec_name),6,f_idx) = bw;
        
    end
    
    %% Save result matrix
    save('GW_test_results_2.mat','metrics');
end

%% Plots
% Load existing file with test results
if strcmp(answer,'No') == 1
    load('GW_test_results_2.mat');
end

% Create figures
mrk_str = {'o','*','s','d'};
ylabel_str = {'\Delta f_{pk} [kHz]','Es [V^{2}.s]',...
    'ToF [\mus]','\Delta f_{ch} [kHz]',...
    ['PSM_{',num2str(psm_ord),'}'],'BW_{FFT} [kHz]'};
title_str = {'204 kHz','349 kHz','486 kHz','619 kHz'};

for m = 1:6
    
    fh1 = figure();
    
    for f_idx = 1:4
        
        % Mean values
        di_mean = [mean(metrics(1:5,m,f_idx)),mean(metrics(6:10,m,f_idx)),...
            mean(metrics(11:14,m,f_idx))];
        % Standard deviation of the mean 
        %(also called standard error of the mean)
        di_stdev = [std(metrics(1:5,m,f_idx))/sqrt(5),...
            std(metrics(6:10,m,f_idx))/sqrt(5),...
            std(metrics(11:14,m,f_idx))/sqrt(4)];
        
        % Define factors for y-axis
        if m == 2 || m == 5
            factor = 1;
        elseif m == 3
           factor = 1e6;
        elseif m == 1 || m == 4 || m == 6
            factor = 1e-3;
        end
        
        if f_idx == 1
            subplot(1,2,1);
        end
        errorbar([1 2 3],di_mean*factor,di_stdev*factor,...
            mrk_str{f_idx},'MarkerFaceColor','auto');
        xlim([0 4]);
        xticks([1 2 3]);
        xlabel('Batch #');
        ylabel(ylabel_str{m});
        grid on
        
        % Define legend
        if f_idx == 1
            hold on
        elseif f_idx == 2
            legend('204 kHz','349 kHz');
            subplot(1,2,2);
        elseif f_idx == 3
            hold on
        elseif f_idx == 4
            legend('486 kHz','619 kHz');
        end
        
    end
end
